#include <iostream>
#include <cstdlib>
using namespace std;

class Lista {
  struct Elem {                        // Element liste.
    int a; Elem* sled = nullptr;
    Elem (int a): a(a) {}
    ~Elem() { delete sled; }
  };
  Elem* prvi = nullptr;                // Pocetak liste.
public:
  class Iter {                         // ITERATOR ZA PROMENLJIVE ZBIRKE:
    Elem* p;                           // Tekuci element.
  public:
    Iter(Elem* p): p(p) {}             // Inicijalizacija.
    Iter& operator++() {               // Prelazak na sledeci element.
      if (p != nullptr) p = p->sled;
      return *this;
    }
    int& operator*() { return p->a; }  // Dohvatanje tekuceg elementa.
    bool operator!=(const Iter& i)     // Da li su razliciti?
      const { return p != i.p; }
    int*const operator->() { return &p->a; } // Adresa tekuceg podatka.
  };

  class C_iter {                       // ITERATOR ZA NEPROMENLJIVE ZBIRKE:
    Elem* p;                           // Tekuci element.
  public:
    C_iter(Elem* p): p(p) {}           // Inicijalizacija.
    C_iter& operator++() {             // Prelazak na sledeci element.
      if (p != nullptr) p = p->sled;
      return *this;
    }
    const int& operator*() const       // Dohvatanje tekuceg elementa.
      { return p->a; }
    bool operator!=(const C_iter& i)   // Da li su razliciti?
      const { return p != i.p; }
    const int*const operator->(){return &p->a;} // Adresa tekuceg podatka.
  };
                                          // METODE ZBIRKE:
  Lista() {                               // Podrazumevani konstruktor.
    Elem* posl = nullptr;
    for (auto x: {1, 2, 3, 4, 5}) {
      posl = (prvi == nullptr ? prvi : posl->sled) = new Elem(x);
    }
  }
  Lista(const Lista&) =delete;            // Nema kopiranja.
  ~Lista() { delete prvi; }               // Destruktor.
  Lista& operator=(const Lista&) =delete; // Nema dodele vrednosti.
                                                    // Stvaranje iteratora:
  Iter begin() { return Iter(prvi); }               // - za promenljive
  const Iter end() { return Iter(nullptr); }        //   zbirke,
  C_iter begin() const { return C_iter(prvi); }     // - za nepromenljive
  const C_iter end()const {return C_iter(nullptr);} //   zbirke.
};

int main() {
  Lista lst;                                // Promenljiva zbirka.
  for (auto x: lst) cout << x << ' ';       // Pise se: 1 2 3 4 5
  cout << endl;
  for (auto x: lst) x *= 2;                 // Zbirka se ne menja.
  for (auto x: lst) cout << x << ' ';       // Pise se: 1 2 3 4 5
  cout << endl;
  for (auto& x: lst) x *= 2;                // Zbirka se menja.
  for (const auto& x: lst) cout << x << ' ';
  cout << endl;                             // Pise se: 2 4 6 8 10
  const Lista clst;                         // Nepromenljiva zbirka.
  for (auto x: clst) cout << x << ' ';      // Pise se: 1 2 3 4 5
  cout << endl;
//  for (auto& x: clst) x *= 2;               // GRESKA: ne moze da se menja.
}

